const equals = Object.is;
const isEmpty = (x) => x === null || x === undefined;
export class TreeNode {
    key;
    data;
    static invertTree(treeList, maxDepth = Number.MAX_SAFE_INTEGER) {
        const results = new Array();
        const invertTree = (treeList, currentDepth) => {
            ++currentDepth;
            if (maxDepth < currentDepth) {
                return false;
            }
            for (const node of treeList) {
                if (!node.children.length || !invertTree(node.children, currentDepth)) {
                    results.push(node);
                }
            }
            return true;
        };
        invertTree(treeList, 0);
        return results;
    }
    static treeFromList(list, keyExpr, parentKeyExpr) {
        const treeNodeCache = new Map();
        const treeNodeListCache = new Map();
        const treeResults = new Array();
        let item;
        let node;
        let parentKey;
        for (let i = 0; i < list.length; ++i) {
            item = list[i];
            node = new TreeNode(keyExpr(item) ?? null, item);
            treeNodeCache.set(node.key, node);
            parentKey = parentKeyExpr(item) ?? null;
            if (isEmpty(parentKey)) {
                treeResults.push(node);
            }
            else if (treeNodeCache.has(parentKey)) {
                const parent = treeNodeCache.get(parentKey);
                parent.addChildNode(node);
            }
            else {
                let parentQueue = treeNodeListCache.get(parentKey);
                if (isEmpty(parentQueue)) {
                    parentQueue = [];
                    treeNodeListCache.set(parentKey, parentQueue);
                }
                parentQueue.push(node);
            }
            const queue = treeNodeListCache.get(node.key);
            if (queue?.length) {
                let child;
                while (queue.length > 0) {
                    child = queue.shift();
                    node.addChildNode(child);
                }
            }
        }
        return treeResults;
    }
    static parseTree(tree) {
        return _parseTree(null, tree);
    }
    _parent;
    get parent() {
        return this._parent?.deref() ?? null;
    }
    set parent(value) {
        const oldParent = this.parent;
        if (oldParent === value) {
            return;
        }
        if (oldParent !== null) {
            oldParent.removeChildNode(this);
        }
        if (value === null) {
            this._parent = null;
        }
        else {
            value.addChildNode(this);
        }
    }
    get root() {
        let parent;
        let node = this;
        while ((parent = node.parent) != null) {
            node = parent;
        }
        return node;
    }
    _children = [];
    get children() {
        return this._children;
    }
    get hasChildren() {
        return this._children.length > 0;
    }
    constructor(key, data) {
        this.key = key;
        this.data = data;
    }
    containsChild(key, maxDepth = Number.MAX_SAFE_INTEGER) {
        return !!this.getChildNode(key, maxDepth);
    }
    getChildNode(key, maxDepth = Number.MAX_SAFE_INTEGER) {
        return _getChildNode(this, key, 1, maxDepth);
    }
    addChildByParent(parentKey, key, data) {
        const child = equals(key, parentKey) ? this : this.getChildNode(parentKey);
        if (child == null) {
            return null;
        }
        const node = new TreeNode(key, data);
        node.parent = child;
        return node;
    }
    addChild(key, data) {
        const node = new TreeNode(key, data);
        node.parent = this;
        return node;
    }
    removeChild(key) {
        const child = this.getChildNode(key);
        if (child != null) {
            child.parent = null;
            return child.data;
        }
        return null;
    }
    removeChildNode(child) {
        if (child != null && child.parent == this) {
            const index = this._children.indexOf(child);
            if (index > -1) {
                this._children.splice(index, 1);
            }
            child._parent = null;
        }
        return this;
    }
    addChildNodeByParentKey(parentKey, child) {
        if (child != null) {
            const node = this.getChildNode(parentKey);
            node?.addChildNode(child);
        }
        return this;
    }
    addChildNode(child) {
        if (child && child.parent !== this) {
            if (this._children.some((x) => equals(x.key, child.key))) {
                throw new Error('Child already exists!');
            }
            this._children.push(child);
            child._parent = new WeakRef(this);
        }
        return this;
    }
    removeChildren(keys) {
        for (const key of keys) {
            this.removeChild(key);
        }
    }
    toList(maxDepth = Number.MAX_SAFE_INTEGER) {
        return _toList(this, 1, maxDepth);
    }
    toJSON() {
        return {
            key: this.key,
            data: this.data,
            children: this._children.map((x) => {
                return x.toJSON();
            }),
        };
    }
}
function _toList(node, currentDepth, maxDepth = Number.MAX_SAFE_INTEGER) {
    if (currentDepth > maxDepth) {
        return null;
    }
    ++currentDepth;
    let results = [node];
    let resultsAux;
    for (const child of node.children) {
        resultsAux = _toList(child, currentDepth, maxDepth);
        if (!resultsAux?.length) {
            continue;
        }
        results = results.concat(resultsAux);
    }
    return results;
}
function _getChildNode(currentNode, key, currentDepth, maxDepth = Number.MAX_SAFE_INTEGER) {
    let auxNode;
    for (const node of currentNode.children) {
        if (equals(node.key, key)) {
            return node;
        }
        if (currentDepth >= maxDepth) {
            continue;
        }
        auxNode = _getChildNode(node, key, currentDepth + 1, maxDepth);
        if (auxNode != null) {
            return auxNode;
        }
    }
    return null;
}
function _parseTree(parent, tree) {
    const results = tree.map((item) => {
        const node = new TreeNode(item.key, item.data);
        if (parent !== null) {
            node.parent = parent;
        }
        if (item.children?.length) {
            _parseTree(node, item.children);
        }
        return node;
    });
    return results;
}
//# sourceMappingURL=tree-node.js.map